;;; -*- lexical-binding: t -*-

(require 'cl-lib)

(defconst abench-lines 100000)

(defun abench-run (modify-before)
  (let ((buffer-undo-list t))
    (when modify-before
      ;; Modify all lines in one sweep without narrowing
      (goto-char (point-min))
      (while (not (eobp))
        (delete-char 1)
        (forward-line)))

    ;; Narrow around each line but do not modify
    (goto-char (point-min))
    (while (not (eobp))
      (narrow-to-region (point) (line-end-position))
      (widen)
      (forward-line))

    (unless modify-before
      ;; Modify all lines in one sweep without narrowing
      (goto-char (point-min))
      (while (not (eobp))
        (delete-char 1)
        (forward-line)))))

(defun abench (modify-before)
  (with-temp-buffer
    ;; Populate buffer
    (insert (mapconcat #'identity (make-list abench-lines "ab\n") nil))
    ;; Run operation
    (garbage-collect)
    (let ((gcs0 gcs-done)
          (gc-e0 gc-elapsed)
          (t0 (float-time)))
      (abench-run modify-before)
      (let ((dt (- (float-time) t0))
            (gcs (- gcs-done gcs0))
            (gc-time (- gc-elapsed gc-e0)))
        (cl-assert
         (equal (buffer-string)
                (mapconcat #'identity (make-list abench-lines "b\n") nil)))
        (format "%.3f s elapsed, %d GCs, %.3f s GC, %.3f s non-GC\n"
                dt gcs gc-time (- dt gc-time))))))

(princ (format "Emacs %s\n" emacs-version))
(princ "Modify after narrowing:  ")
(princ (abench nil))
(princ "Modify before narrowing: ")
(princ (abench t))
